'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.getEventName = exports.detectIsEvent = exports.delegateEvent = exports.SyntheticEvent = void 0;
const core_1 = require('@dark-engine/core');
class SyntheticEvent {
  type = '';
  sourceEvent = null;
  target = null;
  propagation = true;
  constructor(options) {
    this.type = options.sourceEvent.type;
    this.sourceEvent = options.sourceEvent;
    this.target = options.target;
  }
  stopPropagation() {
    this.propagation = false;
    this.sourceEvent.stopPropagation();
  }
  preventDefault() {
    this.sourceEvent.preventDefault();
  }
  getPropagation() {
    return this.propagation;
  }
}
exports.SyntheticEvent = SyntheticEvent;
function delegateEvent(options) {
  const { target, eventName, handler } = options;
  const eventsMap = core_1.eventsStore.get();
  const handlerMap = eventsMap.get(eventName);
  if (!handlerMap) {
    const rootHandler = event => {
      const fireEvent = eventsMap.get(eventName).get(event.target);
      const target = event.target;
      let syntheticEvent = null;
      if ((0, core_1.detectIsFunction)(fireEvent)) {
        syntheticEvent = new SyntheticEvent({
          sourceEvent: event,
          target,
        });
        fireEvent(syntheticEvent);
      }
      if (syntheticEvent ? syntheticEvent.getPropagation() : target.parentElement) {
        target.parentElement.dispatchEvent(new event.constructor(event.type, event));
      }
    };
    eventsMap.set(eventName, new WeakMap([[target, handler]]));
    document.addEventListener(eventName, rootHandler, true);
    core_1.eventsStore.addUnsubscriber(() => document.removeEventListener(eventName, rootHandler, true));
  } else {
    handlerMap.set(target, handler);
  }
}
exports.delegateEvent = delegateEvent;
const detectIsEvent = attrName => attrName.startsWith('on');
exports.detectIsEvent = detectIsEvent;
const getEventName = attrName => attrName.slice(2, attrName.length).toLowerCase();
exports.getEventName = getEventName;
//# sourceMappingURL=events.js.map
