import { createComponent, detectIsComponent, useMemo } from '@dark-engine/core';
const $$portal = Symbol('portal');
function createPortal(slot, container) {
  if (!(container instanceof Element)) {
    throw new Error(`[Dark]: createPortal receives only Element as container!`);
  }
  return Portal({ [$$portal]: container, slot });
}
const Portal = createComponent(
  ({ slot, ...rest }) => {
    useMemo(() => (rest[$$portal].innerHTML = ''), []);
    return slot;
  },
  { token: $$portal },
);
const detectIsPortal = instance => detectIsComponent(instance) && instance.token === $$portal;
const getPortalContainer = factory => (detectIsPortal(factory) ? factory.props[$$portal] : null);
function unmountPortal(fiber) {
  const container = getPortalContainer(fiber.instance);
  if (container) {
    container.innerHTML = '';
  }
}
export { createPortal, detectIsPortal, getPortalContainer, unmountPortal };
//# sourceMappingURL=portal.js.map
