import {
  ROOT,
  Fiber,
  EffectTag,
  platform,
  flatten,
  detectIsUndefined,
  TagVirtualNode,
  rootStore,
  wipRootStore,
  currentRootStore,
  nextUnitOfWorkStore,
  fiberMountStore,
  TaskPriority,
  createReplacer,
  isInsertionEffectsZone,
  isLayoutEffectsZone,
  isHydrateZone,
} from '@dark-engine/core';
import { createNativeElement, applyCommit, finishCommitWork } from '../dom';
import { detectIsPortal, unmountPortal } from '../portal';
import { scheduleCallback, shouldYeildToHost } from '../scheduler';
let isInjected = false;
const roots = new Map();
function inject() {
  platform.createNativeElement = createNativeElement;
  platform.requestAnimationFrame = requestAnimationFrame.bind(this);
  platform.cancelAnimationFrame = cancelAnimationFrame.bind(this);
  platform.scheduleCallback = scheduleCallback;
  platform.shouldYeildToHost = shouldYeildToHost;
  platform.applyCommit = applyCommit;
  platform.finishCommitWork = finishCommitWork;
  platform.detectIsDynamic = () => true;
  platform.detectIsPortal = detectIsPortal;
  platform.unmountPortal = unmountPortal;
  platform.restart = () => {};
  isInjected = true;
}
function render(element, container, hydrate = false) {
  !isInjected && inject();
  if (!(container instanceof Element)) {
    throw new Error(`[Dark]: render receives only Element as container!`);
  }
  const isMounted = !detectIsUndefined(roots.get(container));
  let rootId = null;
  if (!isMounted) {
    rootId = roots.size;
    roots.set(container, rootId);
    if (!hydrate) {
      container.innerHTML = '';
    }
  } else {
    rootId = roots.get(container);
  }
  if (isInsertionEffectsZone.get(rootId)) return;
  const callback = () => {
    rootStore.set(rootId);
    const currentRoot = currentRootStore.get();
    const isUpdate = Boolean(currentRoot);
    const fiber = new Fiber().mutate({
      nativeElement: container,
      instance: new TagVirtualNode(ROOT, {}, flatten([element || createReplacer()])),
      alternate: currentRoot,
      effectTag: isUpdate ? EffectTag.UPDATE : EffectTag.CREATE,
    });
    fiberMountStore.reset();
    wipRootStore.set(fiber);
    isHydrateZone.set(hydrate);
    nextUnitOfWorkStore.set(fiber);
  };
  platform.scheduleCallback(callback, {
    priority: TaskPriority.NORMAL,
    forceSync: hydrate || isLayoutEffectsZone.get(),
  });
}
export { render, roots };
//# sourceMappingURL=render.js.map
